from ._AckMessage import AckMessage
from ._DataType import DataType
from ._DefaultToken import DefaultToken
from ._Event import Event
from ._JsonSwaggerMessage import JsonSwaggerMessage
from ._Message import Message
from ._MessageBus import MessageBus
from ._MessageBusException import MessageBusException
from ._MessageBusExceptionType import MessageBusExceptionType
from ._NakMessage import NakMessage
from ._SubscriptionInfo import SubscriptionInfo
from ._SwaggerMessage import SwaggerMessage
from ._TokenCreator import TokenCreator
from ._TokenPriority import TokenPriority
from ._HttpRequestMessage import HttpRequestMessage
from ._HttpResponseMessage import HttpResponseMessage
from ._JsonHttpResponseMessage import JsonHttpResponseMessage
from ._InternalErrorMessage import InternalErrorMessage


__all__ = [
    "Message",
    "AckMessage",
    "NakMessage",
    "SwaggerMessage",
    "JsonSwaggerMessage",
    "MessageBus",
    "Event",
    "TokenCreator",
    "TokenPriority",
    "DataType",
    "DefaultToken",
    "MessageBusExceptionType",
    "MessageBusException",
    "SubscriptionInfo",
    "HttpRequestMessage",
    "HttpResponseMessage",
    "JsonHttpResponseMessage",
    "InternalErrorMessage"
]



# vat.py - functions for handling German VAT numbers
#
# Copyright (C) 2012, 2013 Arthur de Jong
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Ust ID Nr. (Umsatzsteur Identifikationnummer, German VAT number).

The number is 10 digits long and uses the ISO 7064 Mod 11, 10 check digit
algorithm.

>>> compact('DE 136,695 976')
'136695976'
>>> validate('DE136695976')
'136695976'
>>> validate('136695978')
Traceback (most recent call last):
    ...
InvalidChecksum: ...
"""

from stdnum.exceptions import *
from stdnum.iso7064 import mod_11_10
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation. This strips the
    number of any valid separators and removes surrounding whitespace."""
    number = clean(number, ' -./,').upper().strip()
    if number.startswith('DE'):
        number = number[2:]
    return number


def validate(number):
    """Check if the number provided is a valid VAT number. This checks the
    length, formatting and check digit."""
    number = compact(number)
    if not isdigits(number) or number[0] == '0':
        raise InvalidFormat()
    if len(number) != 9:
        raise InvalidLength()
    mod_11_10.validate(number)
    return number


def is_valid(number):
    """Check if the number provided is a valid VAT number. This checks the
    length, formatting and check digit."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False



from application.service import LoaderUpdaterService


class LoaderUpdaterInteractor(object):
    def __init__(
        self,                       # type: LoaderUpdaterInteractor
        loader_updater_service,     # type: LoaderUpdaterService
    ):
        # type: (...) -> None

        self.loader_updater_service = loader_updater_service

    def update_loaders(
        self,   # type: LoaderUpdaterInteractor
    ):
        # type: (...) -> None

        return self.loader_updater_service.update_loaders(restart_hv=True)



import json

from sysactions import show_messagebox
from typing import (
    Dict,
    Optional,
    List,
)

import pyscripts

from msgbus import (
    FM_PARAM,
    TK_SYS_ACK,
    MBMessage,
)
from bustoken import TK_PROD_ORDER_PICKLIST


mb_context = pyscripts.mbcontext


def get_pick_list_orders(
    pos_id,             # type: str
    orders_list,        # type: List[str]
):
    # type: (...) -> Optional[Dict]

    try:
        data = {
            "posId": pos_id,
            "ordersList": orders_list,
        }

        response = _send_message(
            data=data,
        )

        if response.token == TK_SYS_ACK:
            return json.loads(
                s=response.data,
            )

    except Exception as ex:
        _show_error_message(
            pos_id=pos_id,
            message=ex.message,
        )

    return None


def print_pick_list_order(
    pos_id,             # type: str
    order_id,           # type: str
    printer_name,       # type: str
    selected_option,    # type: str
):
    # type: (...) -> bool

    try:
        data = {
            "posId": pos_id,
            "orderId": order_id,
            "selectedOption": selected_option,
            "printerName": printer_name,
        }
        response = _send_message(
            data=data,
        )
        if response.token != TK_SYS_ACK:
            _show_error_message(
                pos_id=pos_id,
                message=response.data,
            )
            return False

        return True
    except Exception as ex:
        _show_error_message(
            pos_id=pos_id,
            message=ex.message,
        )
        return False


def _send_message(
    data,   # type: Dict
):
    # type: (...) -> MBMessage

    data = json.dumps(
        obj=data,
    )

    return mb_context.MB_EasySendMessage(
        dest_name="ProductionSystem",
        token=TK_PROD_ORDER_PICKLIST,
        format=FM_PARAM,
        data=data,
        timeout=(10 * 1000000)
    )


def _show_error_message(
    pos_id,     # type: str
    message,    # type: str
):
    # type: (...) -> None

    message = "$ERROR_REPRINT_PICKLIST|{}".format(
        message
    )

    show_messagebox(
        posid=pos_id,
        message=message,
        title="$ERROR",
    )



from production.box import ProductCodeFilterBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class ProductCodeFilterBoxBuilder(BoxBuilder):
    def __init__(self, loggers):
        super(ProductCodeFilterBoxBuilder, self).__init__(loggers)

    def build(self, config):
        # type: (BoxConfiguration) -> ProductCodeFilterBox
        return ProductCodeFilterBox(config.name,
                                    config.sons,
                                    self.get_extra(config, "ProductCodes", None),
                                    self.get_extra(config, "KeepChildren", False),
                                    self.get_logger(config))



import json
from typing import Any, List

from helper import json_serialize
from messagebus import Message, DefaultToken, DataType
from messageprocessorutil import UuidMessageProcessor

from dailygoals_services.interactor.sales_challenge import GetSalesChallengeInteractor

from externals.util import LoggerUtil

logger = LoggerUtil.get_logger_name(__name__)


class GetSalesChallengeProcessor(UuidMessageProcessor):
    def __init__(self, interactor):
        # type: (GetSalesChallengeInteractor) -> None

        super(GetSalesChallengeProcessor, self).__init__()

        self.interactor = interactor

    def parse_data(self, data, event_type):
        # type: (Any, str) -> Any

        parsed_data = json.loads(data)
        limit = int(parsed_data.get("limit")) if parsed_data.get("limit") else None

        return limit

    def call_business(self, params):
        # type: (Any) -> List

        return self.interactor.get_sales_challenge(params)

    def format_response(self, message_bus, message, event, request, response):
        logger.info("Goals updated for request")
        reply_message = Message(DefaultToken.TK_SYS_ACK, DataType.json, json_serialize(response))
        if message:
            message_bus.reply_message(message, reply_message)
        elif event:
            message_bus.reply_event(event, reply_message)

    def format_exception(self, message_bus, message, event, request, exception):
        exception_name = str(type(exception).__name__)
        logger.info("Error updating total sold from request: {}".format(exception_name))
        self._reply_exception(event, exception, message, message_bus)

    def format_parse_exception(self, message_bus, message, event, data, exception):
        logger.info("Error getting parsing request: {}".format(str(exception)))
        self._reply_exception(event, exception, message, message_bus)

    @staticmethod
    def _reply_exception(event, exception, message, message_bus):
        reply_message = Message(DefaultToken.TK_SYS_NAK, DataType.param, str(exception))
        if message:
            reply_message = Message(DefaultToken.TK_SYS_NAK, DataType.param, str(type(exception).__name__))
            message_bus.reply_message(message, reply_message)
        elif event:
            message_bus.reply_event(event, reply_message)



from xml.etree import cElementTree as eTree

from helper import remove_accents
from typing import (
    Dict,
    List,
    Optional,
    Any,
)

import sysactions

from helper import translate_pos_label
from old_helper import F, get_sale_line_priced_items, round_half_away_from_zero

from common import PrinterCommon
from printer import simplify_sale_line_elements
from _AposA8PrintLine import AposA8PrintLine
from _AposA8Align import AposA8Align
from _AposA8CommandType import AposA8CommandType
from _AposA8Constants import AposA8Constants
from _AposA8FontSize import AposA8FontSize
from services import order_xml_has_valid_fee

APOS_DASH = AposA8PrintLine(AposA8Align.CENTER, AposA8Constants.SEPARATOR_DASH, AposA8CommandType.PRINT_TEXT)
APOS_NEWLINE = AposA8PrintLine(AposA8Align.CENTER, "", AposA8CommandType.PRINT_TEXT)
APOS_HASH = AposA8PrintLine(AposA8Align.CENTER, AposA8Constants.SEPARATOR_HASH, AposA8CommandType.PRINT_TEXT)


def add_loyalty_custom_text(report, order, custom_text):
    # type: (list, eTree.Element, dict) -> None
    loyalty_user_name = order.find("CustomOrderProperties/OrderProperty[@key='CUSTOMER_NAME']")
    loyalty_user_cpf = order.find("CustomOrderProperties/OrderProperty[@key='LOYALTY_SYSTEM_USER_CPF']")

    if loyalty_user_name is not None and loyalty_user_cpf is not None:
        name_and_cpf = custom_text["name_cpf"].format(nome_cliente=loyalty_user_name.get("value"),
                                                      cpf_cliente=loyalty_user_cpf.get("value"))
        report.append(AposA8PrintLine(AposA8Align.CENTER, name_and_cpf, AposA8CommandType.PRINT_TEXT))
    elif loyalty_user_cpf is not None:
        cpf = custom_text["cpf"].format(cpf_cliente=loyalty_user_cpf.get("value"))
        report.append(AposA8PrintLine(AposA8Align.CENTER, cpf, AposA8CommandType.PRINT_TEXT))
    else:
        non_participating = custom_text["non_participating"]
        report.append(AposA8PrintLine(AposA8Align.CENTER, non_participating, AposA8CommandType.PRINT_TEXT))

    report.append(APOS_NEWLINE)
    report.append(APOS_DASH)
    report.append(APOS_NEWLINE)


def append_items_header(
    report,         # type: List[AposA8PrintLine]
    font_size,      # type: str
):
    # type: (...) -> None

    cod = _get_label_item(
        text='COD',
        column=0,
        font_size=font_size,
    )

    descr = _get_label_item(
        text='DESCR',
        column=1,
        font_size=font_size,
    )

    qtd = _get_label_item(
        text='QTD',
        column=2,
        font_size=font_size,
    )

    un = _get_label_item(
        text='UN',
        column=3,
        font_size=font_size,
    )

    vl_unit = _get_label_item(
        text='VL UNIT',
        column=4,
        font_size=font_size,
        align='>'

    )

    total = _get_label_item(
        text='TOTAL',
        column=5,
        font_size=font_size,
        align='>'

    )

    if font_size == AposA8FontSize.SMALL:
        header_lines = '{}{}{}{}{}{}'.format(cod, descr, qtd, un, vl_unit, total)
        report.append(
            get_text_line(
                text=header_lines,
                font_size=font_size,
            )
        )
        return

    header_lines = '{}{}'.format(cod, descr)
    report.append(
        get_text_line(
            text=header_lines,
            font_size=font_size,
            align=AposA8Align.LEFT,
        )
    )

    vl_unit = format_text(
        text=vl_unit.strip(),
        width=10,
        align='>',
    )

    total = format_text(
        text=total.strip(),
        width=13,
        align='>',
    )

    header_lines = '{}{}{}{}'.format(qtd, un, vl_unit, total)
    report.append(
        get_text_line(
            text=header_lines,
            font_size=font_size,
            align=AposA8Align.LEFT,
        )
    )


def _get_label_item(
    text,           # type: str
    column,         # type: int
    font_size,      # type: str
    align='<',      # type: str
):
    # type: (...) -> str

    get_columns_size = _get_columns_size(
        column=column,
        font_size=font_size,
    )

    return format_text(
        text=text[:get_columns_size],
        align=align,
        width=get_columns_size,
    )


def format_text(
    text,           # type: str
    width,          # type: int
    align='<',      # type: str
):
    # type: (...) -> str

    text_len = len(remove_accents(text=text))

    if align == '<':
        text += (" " * (width - text_len))
    else:
        text = (" " * (width - text_len)) + text

    return text


def _get_columns_size(
    column,         # type: int
    font_size,      # type: str
):
    # type: (...) -> int

    columns_and_size = {
        0: 8,
        1: 14,
        2: 7,
        3: 2,
        4: 8,
        5: 9,
    }

    if font_size != AposA8FontSize.SMALL:
        columns_and_size = {
            0: 8,
            1: AposA8Constants.COLS_FONT_LARGE - 8,
            2: 6,
            3: 3,
            4: 18,
            5: 18,
        }

    return columns_and_size[column]


def append_and_count_sale_items(
    report,             # type: List[AposA8PrintLine]
    order,              # type: eTree.Element
    printer_common,     # type: PrinterCommon
    font_size,          # type: str
):
    # type: (...) -> int

    items_amount = 0

    sale_lines = simplify_sale_line_elements(order.findall("SaleLine"))
    sale_lines = get_sale_line_priced_items(sale_lines)

    for line in sale_lines:
        line_params = _get_line_params(line)
        item_price, multiplied_qty, name, part_code, level = line_params
        measure_unit = line.get("measureUnit") if "measureUnit" in line.attrib else "un"
        items_amount += 1
        quantity = F(multiplied_qty)
        if order_xml_has_valid_fee(order_xml=order):
            fee_amount = float(line.get("feeIncludedInPrice") or 0.0)
            item_price = round_half_away_from_zero(F(item_price) - fee_amount, 2)
        item_value = round_half_away_from_zero(F(item_price) / quantity, 2)
        _append_sale_item(
            report=report,
            part_code=part_code,
            name=name,
            quantity=quantity,
            measure_unit=measure_unit,
            item_value=item_value,
            item_price=item_price,
            font_size=font_size,
        )

    if float(order.get("tip", 0)) > 0:
        tips_lines = printer_common.get_tip_order(order, is_mobile=True)
        tips_lines = tips_lines.split("\n")
        for line in tips_lines:
            split_line = line.split("|")
            if not split_line[0]:
                continue

            items_amount += 1
            _append_tip(
                report=report,
                split_line=split_line,
                font_size=font_size,
            )

    return items_amount


def _append_tip(
    report,         # type: List[AposA8PrintLine]
    split_line,     # type: List[str]
    font_size,      # type: str
):
    # type: (...) -> None

    part_code = split_line[0].strip()
    name = split_line[1].strip()
    quantity = float(split_line[2].strip() or '0.00')
    measure_unit = split_line[3].strip()
    item_value = float(split_line[4].strip() or '0.00')
    item_price = float(split_line[5].strip() or '0.00')

    _append_sale_item(
        report=report,
        part_code=part_code,
        name=name,
        quantity=quantity,
        measure_unit=measure_unit,
        item_value=item_value,
        item_price=item_price,
        font_size=font_size,
    )


def _append_sale_item(
    report,         # type: List[AposA8PrintLine]
    part_code,      # type: str
    name,           # type: str
    quantity,       # type: float
    measure_unit,   # type: str
    item_value,     # type: float
    item_price,     # type: float
    font_size,      # type: str
):

    part_code = _get_label_item(
        text=part_code,
        column=0,
        font_size=font_size,
    )

    name = _get_label_item(
        text=name,
        column=1,
        font_size=font_size,
    ).encode("utf-8")

    quantity = _get_label_item(
        text="{:.3f}".format(quantity),
        column=2,
        font_size=font_size
    )

    measure_unit = _get_label_item(
        text=measure_unit.upper(),
        column=3,
        font_size=font_size,
    )

    item_value = _get_label_item(
        text="{:.2f}".format(item_value),
        column=4,
        font_size=font_size,
        align='>'
    )

    item_price = _get_label_item(
        text="{:.2f}".format(F(item_price)),
        column=5,
        font_size=font_size,
        align='>'
    )

    if font_size == AposA8FontSize.SMALL:
        _append_item_small_font(
            report=report,
            font_size=font_size,
            part_code=part_code,
            name=name,
            quantity=quantity,
            measure_unit=measure_unit,
            item_price=item_price,
            item_value=item_value,
        )
    else:
        _append_item_large_font(
            report=report,
            font_size=font_size,
            part_code=part_code,
            name=name,
            quantity=quantity,
            measure_unit=measure_unit,
            item_price=item_price,
            item_value=item_value,
        )


def _append_item_small_font(
    report,             # type: List[AposA8PrintLine]
    font_size,          # type: str
    part_code,          # type: str
    name,               # type: bytes
    quantity,           # type: str
    measure_unit,       # type: str
    item_value,         # type: str
    item_price,         # type: str
):
    # type: (...) -> None

    item_line = '{}{}{}{}{}{}'.format(part_code, name, quantity, measure_unit, item_value, item_price)
    report.append(
        get_text_line(
            text=item_line,
            font_size=font_size,
        )
    )


def _append_item_large_font(
    report,             # type: List[AposA8PrintLine]
    font_size,          # type: str
    part_code,          # type: str
    name,               # type: bytes
    quantity,           # type: str
    measure_unit,       # type: str
    item_value,         # type: str
    item_price,         # type: str
):
    # type: (...) -> None

    item_line = '{}{}'.format(part_code, name[:24])
    report.append(
        get_text_line(
            text=item_line,
            font_size=font_size,
            align=AposA8Align.LEFT,
        )
    )

    quantity = format_text(
        text=quantity.rstrip(),
        width=7,
        align='>'
    )

    measure_unit = format_text(
        text=measure_unit,
        width=0,
    )

    item_value = format_text(
        text=item_value.strip(),
        width=9,
        align='>',
    )

    item_price = format_text(
        text=item_price.strip(),
        width=13,
        align='>',
    )

    item_line = '{}{}{}{}'.format(quantity, measure_unit, item_value, item_price)
    report.append(
        get_text_line(
            text=item_line,
            font_size=font_size,
            align=AposA8Align.LEFT,
        )
    )

    report.append(APOS_NEWLINE)


def _get_line_params(
        line    # type: eTree
):
    # type: (...) -> map

    return map(
                line.get,
                (
                    "itemPrice",
                    "multipliedQty",
                    "productName",
                    "partCode",
                    "level",
                )
        )


def append_tax_data(
    report,             # type: List[AposA8PrintLine]
    total_taxes,        # type: Dict
    font_size,          # type: str
):
    # type: (...) -> None

    tax_amount = float(total_taxes['nacionalfederal']['value']) + float(total_taxes['estadual']['value'])
    tax_text = "%-20s %7.2f" % ("Valor aproximado dos tributos deste cupom", tax_amount)
    split_lines(
        report=report,
        text=tax_text,
        font_size=font_size,
    )

    tax_federal_amount = "R$%7.2f" % total_taxes['nacionalfederal']['value']
    tax_federal_percentage = "(%.2f%%)" % total_taxes['nacionalfederal']['percent']
    federal_taxes = "Fed = {} {}".format(tax_federal_amount, tax_federal_percentage)

    tax_state_amount = "R$%7.2f" % total_taxes['estadual']['value']
    tax_state_percentage = "(%.2f%%)" % total_taxes['estadual']['percent']
    state_taxes = "Est = {} {}".format(tax_state_amount, tax_state_percentage)

    taxes = ["{}, {}".format(federal_taxes, state_taxes)]

    if font_size != AposA8FontSize.SMALL:
        taxes = [federal_taxes, state_taxes]

    for tax in taxes:
        report.append(
            get_text_line(
                text=tax,
                font_size=font_size,
            )
        )

    text = "(Conforme Lei Fed. 12.741/2012)"
    report.append(
        get_text_line(
            text=text,
            font_size=font_size,
        )
    )


def append_coupon_fiscal_parameters(
    coupon_fiscal_parameters,   # type: Dict[str, bool]
    report,                     # type: List
    order,                      # type: eTree.Element
    font_size,                  # type: str
):
    # type: (...) -> None

    if not any(coupon_fiscal_parameters.values()):
        return

    coupon_fiscal_text = []
    if coupon_fiscal_parameters["operator"]:
        user_id = order.get("sessionId").split("user=")[1].split(",")[0]
        coupon_fiscal_text.append("Operador: {}".format(user_id))

    if coupon_fiscal_parameters["order_or_table"]:
        order_id = order.get("orderId")
        table_property = order.find("CustomOrderProperties/OrderProperty[@key='TABLE_NAME']")
        table_name = table_property.get("value") if table_property is not None else ''
        table_name = table_name[:15] + "..." if len(table_name) > 15 else table_name
        if table_name:
            coupon_fiscal_text.append("Pedido/Mesa: {}/{}".format(order_id[-4:], table_name))
        else:
            coupon_fiscal_text.append("Pedido: {}".format(order_id[-4:]))

    if coupon_fiscal_parameters["pos_id"]:
        pos_id = order.get("posId")
        model = sysactions.get_model(posid=pos_id)
        pos = translate_pos_label(model=model)
        coupon_fiscal_text.append("{} ID: {}".format(pos, pos_id))

    if len(coupon_fiscal_text) > 0:
        coupon_fiscal_text = " / ".join(coupon_fiscal_text) + "\n"
        split_lines(
            report=report,
            text=coupon_fiscal_text,
            font_size=font_size,
        )


def build_call_id(
    model,                          # type: eTree
    report,                         # type: List
    order,                          # type: eTree
    font_size_withdrawal_panel,     # type: str
    printer_common,                 # type: PrinterCommon
    order_ref_ids,                  # type: list
    position="between",             # type: Optional[str]
):
    # type: (...) -> None

    content = printer_common.get_coupon_call_id(
        order=order,
        order_ref_ids=order_ref_ids,
        customer_name_limit=21,
    )
    if not content:
        return

    is_table_order = order.find(".//OrderProperty[@key='TABLE_ID']") is not None
    if is_table_order:
        return

    if position == "between":
        report.append(APOS_HASH)

    panel_call_text = printer_common.get_coupon_call_text(
        model=model,
    )
    panel_call_text = panel_call_text.decode("cp860").encode("utf-8")

    split_lines(
        report=report,
        text=panel_call_text,
        font_size=font_size_withdrawal_panel,
        bold=True,
    )

    report.append(
        AposA8PrintLine(
            align=AposA8Align.CENTER,
            text=content.upper(),
            command_type=AposA8CommandType.PRINT_TEXT,
            font_size=font_size_withdrawal_panel,
            bold=True,
        )
    )

    report.append(APOS_HASH)

    if position == "before":
        report.append(APOS_NEWLINE)


def append_lines_from_array(
    report,         # type: List[AposA8PrintLine]
    lines,          # type: Optional[Any]
    font_size,      # type: str
    bold=False,     # type: Optional[bool]
):
    # type: (...) -> None

    for line in lines:
        report.append(
            get_text_line(
                text=line,
                font_size=font_size,
                bold=bold,
            )
        )


def split_lines(
    report,             # type: List[AposA8PrintLine]
    text,               # type: str
    font_size,          # type: str
    bold=False,         # type: Optional[bool]
    separator=" ",      # type: Optional[str]
):
    # type: (...) -> None

    lines = [text]
    is_small_font = font_size == AposA8FontSize.SMALL
    max_line_size = AposA8Constants.COLS if is_small_font else AposA8Constants.COLS_FONT_LARGE

    if max_line_size is not None and len(text) > max_line_size:
        lines = []
        text_list = text.split(separator)
        new_text = ""
        for index, line in enumerate(text_list):
            current_separator = "" if index == 0 else separator
            formatted_line = "{}{}{}".format(new_text, current_separator, line).strip()
            if len(formatted_line) <= max_line_size:
                new_text = formatted_line
                if index == len(text_list) - 1:
                    lines.append(new_text)
            else:
                lines.append(new_text)
                new_text = line.strip()
                if index == len(text_list) - 1:
                    lines.append(new_text)

    for line in lines:
        report.append(
            get_text_line(
                text=line,
                font_size=font_size,
                bold=bold,
            )
        )


def get_text_line(
    text,           # type: str
    font_size,      # type: str
    align=None,     # type: Optional[int]
    bold=False,     # type: Optional[bool]
):
    # type: (...) -> AposA8PrintLine

    align = align if align is not None else AposA8Align.CENTER

    return AposA8PrintLine(
        align=align,
        text=text,
        command_type=AposA8CommandType.PRINT_TEXT,
        font_size=font_size,
        bold=bold,
    )



from ._TranslatedMessage import TranslatedMessage


class CustomError(object):
    def __init__(
        self,     # type: CustomError
        message,  # type: TranslatedMessage
    ):
        # type: (...) -> None

        self.message = message



# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import struct

import dns.exception
import dns.rdata
import dns.name


class PX(dns.rdata.Rdata):

    """PX record.

    @ivar preference: the preference value
    @type preference: int
    @ivar map822: the map822 name
    @type map822: dns.name.Name object
    @ivar mapx400: the mapx400 name
    @type mapx400: dns.name.Name object
    @see: RFC 2163"""

    __slots__ = ['preference', 'map822', 'mapx400']

    def __init__(self, rdclass, rdtype, preference, map822, mapx400):
        super(PX, self).__init__(rdclass, rdtype)
        self.preference = preference
        self.map822 = map822
        self.mapx400 = mapx400

    def to_text(self, origin=None, relativize=True, **kw):
        map822 = self.map822.choose_relativity(origin, relativize)
        mapx400 = self.mapx400.choose_relativity(origin, relativize)
        return '%d %s %s' % (self.preference, map822, mapx400)

    @classmethod
    def from_text(cls, rdclass, rdtype, tok, origin=None, relativize=True):
        preference = tok.get_uint16()
        map822 = tok.get_name()
        map822 = map822.choose_relativity(origin, relativize)
        mapx400 = tok.get_name(None)
        mapx400 = mapx400.choose_relativity(origin, relativize)
        tok.get_eol()
        return cls(rdclass, rdtype, preference, map822, mapx400)

    def to_wire(self, file, compress=None, origin=None):
        pref = struct.pack("!H", self.preference)
        file.write(pref)
        self.map822.to_wire(file, None, origin)
        self.mapx400.to_wire(file, None, origin)

    @classmethod
    def from_wire(cls, rdclass, rdtype, wire, current, rdlen, origin=None):
        (preference, ) = struct.unpack('!H', wire[current: current + 2])
        current += 2
        rdlen -= 2
        (map822, cused) = dns.name.from_wire(wire[: current + rdlen],
                                             current)
        if cused > rdlen:
            raise dns.exception.FormError
        current += cused
        rdlen -= cused
        if origin is not None:
            map822 = map822.relativize(origin)
        (mapx400, cused) = dns.name.from_wire(wire[: current + rdlen],
                                              current)
        if cused != rdlen:
            raise dns.exception.FormError
        if origin is not None:
            mapx400 = mapx400.relativize(origin)
        return cls(rdclass, rdtype, preference, map822, mapx400)

    def choose_relativity(self, origin=None, relativize=True):
        self.map822 = self.map822.choose_relativity(origin, relativize)
        self.mapx400 = self.mapx400.choose_relativity(origin, relativize)



# oss.py - functions for handling European VAT numbers
# coding: utf-8
#
# Copyright (C) 2023 Arthur de Jong
# Copyright (C) 2023 Sergi Almacellas Abellana
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""OSS (European VAT on e-Commerce - One Stop Shop).

This covers number that have been issued under the non-union scheme and the
import scheme of the European VAT on e-Commerce one stop shop. Numbers under
the non-union scheme are assigned to foreign companies providing digital
services to European Union consumers. Numbers under the import scheme are
assigned for importing goods from a third country (through an intermediary).

This is also called MOSS (mini One Stop Shop) and VoeS (VAT on e-Services).
The number under the import scheme is also called IOSS (Import One Stop Shop)
number.

Numbers under the non-union scheme are in the format EUxxxyyyyyz. For the
import scheme the format is IMxxxyyyyyyz. An intermediary will also get an
number in the format INxxxyyyyyyz but that may not be used as VAT
identification number.

There appears to be a check digit algorithm but the FITSDEV2-SC12-TS-Mini1SS
(Mini-1SS – Technical Specifications) document that describes the algorithm
appears to not be publicly available.

More information:

* https://vat-one-stop-shop.ec.europa.eu/one-stop-shop/register-oss_en
* https://europa.eu/youreurope/business/taxation/vat/vat-digital-services-moss-scheme/index_en.htm
* https://eur-lex.europa.eu/legal-content/EN/TXT/HTML/?uri=CELEX:32002L0038

>>> validate('EU 372022452')
'EU372022452'
"""


from stdnum.exceptions import *
from stdnum.util import clean, isdigits


ISO_3166_1_MEMBER_STATES = (
    '040',  # Austria
    '056',  # Belgium
    '100',  # Bulgaria
    '191',  # Croatia
    '196',  # Cyprus
    '203',  # Czechia
    '208',  # Denmark
    '233',  # Estonia
    '246',  # Finland
    '250',  # France
    '276',  # Germany
    '300',  # Greece
    '348',  # Hungary
    '372',  # Ireland
    '380',  # Italy
    '428',  # Latvia
    '440',  # Lithuania
    '442',  # Luxembourg
    '470',  # Malta
    '528',  # Netherland
    '616',  # Poland
    '620',  # Portugal
    '642',  # Romania
    '703',  # Slovakia
    '705',  # Slovenia
    '724',  # Spain
    '752',  # Sweden
    '900',  # N. Ireland
)
"""The collection of member state codes (for MSI) that may make up a VAT number."""


def compact(number):
    """Compact European VAT Number"""
    return clean(number, ' -').upper().strip()


def validate(number):
    """Validate European VAT Number"""
    number = compact(number)
    if number.startswith('EU'):
        if len(number) != 11:
            raise InvalidLength()
    elif number.startswith('IM'):
        if len(number) != 12:
            raise InvalidLength()
    else:
        raise InvalidComponent()
    if not isdigits(number[2:]):
        raise InvalidFormat()
    if number[2:5] not in ISO_3166_1_MEMBER_STATES:
        raise InvalidComponent()
    return number


def is_valid(number):
    """Check if the number is a valid VAT number. This performs the
    country-specific check for the number."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False



import pyscripts

from mbcontextmessagehandler import MbContextMessageBus
from pos_api import PosService


def get_pos_service(
    pos_id,     # type: int
):
    # type: (...) -> PosService

    return PosService(
        message_bus=MbContextMessageBus(mbcontext=pyscripts.mbcontext),
        pos_id=pos_id,
    )



class ConfigInfo(object):
    def __init__(
        self,
        database_path: str,
        component_name: str = "EventManager",
        max_worker_threads: int = 10,
        maintenance_interval: int = 30 * 60 * 1000,
        acked_events_ttl: int = 7,
        abandoned_events_ttl: int = 30,
    ):
        self.database_path = database_path
        self.component_name = component_name

        self.max_worker_threads = max_worker_threads
        self.maintenance_interval = maintenance_interval
        self.acked_events_ttl = acked_events_ttl
        self.abandoned_events_ttl = abandoned_events_ttl



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# DO NOT EDIT. THIS FILE WAS AUTOGENERATED FROM INSTRUMENTATION PACKAGES.
# RUN `python scripts/generate_instrumentation_bootstrap.py` TO REGENERATE.

libraries = {
    "aio_pika": {
        "library": "aio_pika >= 7.2.0, < 10.0.0",
        "instrumentation": "opentelemetry-instrumentation-aio-pika==0.41b0",
    },
    "aiohttp": {
        "library": "aiohttp ~= 3.0",
        "instrumentation": "opentelemetry-instrumentation-aiohttp-client==0.41b0",
    },
    "aiopg": {
        "library": "aiopg >= 0.13.0, < 2.0.0",
        "instrumentation": "opentelemetry-instrumentation-aiopg==0.41b0",
    },
    "asgiref": {
        "library": "asgiref ~= 3.0",
        "instrumentation": "opentelemetry-instrumentation-asgi==0.41b0",
    },
    "asyncpg": {
        "library": "asyncpg >= 0.12.0",
        "instrumentation": "opentelemetry-instrumentation-asyncpg==0.41b0",
    },
    "boto": {
        "library": "boto~=2.0",
        "instrumentation": "opentelemetry-instrumentation-boto==0.41b0",
    },
    "boto3": {
        "library": "boto3 ~= 1.0",
        "instrumentation": "opentelemetry-instrumentation-boto3sqs==0.41b0",
    },
    "botocore": {
        "library": "botocore ~= 1.0",
        "instrumentation": "opentelemetry-instrumentation-botocore==0.41b0",
    },
    "cassandra-driver": {
        "library": "cassandra-driver ~= 3.25",
        "instrumentation": "opentelemetry-instrumentation-cassandra==0.41b0",
    },
    "scylla-driver": {
        "library": "scylla-driver ~= 3.25",
        "instrumentation": "opentelemetry-instrumentation-cassandra==0.41b0",
    },
    "celery": {
        "library": "celery >= 4.0, < 6.0",
        "instrumentation": "opentelemetry-instrumentation-celery==0.41b0",
    },
    "confluent-kafka": {
        "library": "confluent-kafka >= 1.8.2, <= 2.2.0",
        "instrumentation": "opentelemetry-instrumentation-confluent-kafka==0.41b0",
    },
    "django": {
        "library": "django >= 1.10",
        "instrumentation": "opentelemetry-instrumentation-django==0.41b0",
    },
    "elasticsearch": {
        "library": "elasticsearch >= 2.0",
        "instrumentation": "opentelemetry-instrumentation-elasticsearch==0.41b0",
    },
    "falcon": {
        "library": "falcon >= 1.4.1, < 4.0.0",
        "instrumentation": "opentelemetry-instrumentation-falcon==0.41b0",
    },
    "fastapi": {
        "library": "fastapi ~= 0.58",
        "instrumentation": "opentelemetry-instrumentation-fastapi==0.41b0",
    },
    "flask": {
        "library": "flask >= 1.0, < 3.0",
        "instrumentation": "opentelemetry-instrumentation-flask==0.41b0",
    },
    "grpcio": {
        "library": "grpcio ~= 1.27",
        "instrumentation": "opentelemetry-instrumentation-grpc==0.41b0",
    },
    "httpx": {
        "library": "httpx >= 0.18.0",
        "instrumentation": "opentelemetry-instrumentation-httpx==0.41b0",
    },
    "jinja2": {
        "library": "jinja2 >= 2.7, < 4.0",
        "instrumentation": "opentelemetry-instrumentation-jinja2==0.41b0",
    },
    "kafka-python": {
        "library": "kafka-python >= 2.0",
        "instrumentation": "opentelemetry-instrumentation-kafka-python==0.41b0",
    },
    "mysql-connector-python": {
        "library": "mysql-connector-python ~= 8.0",
        "instrumentation": "opentelemetry-instrumentation-mysql==0.41b0",
    },
    "mysqlclient": {
        "library": "mysqlclient < 3",
        "instrumentation": "opentelemetry-instrumentation-mysqlclient==0.41b0",
    },
    "pika": {
        "library": "pika >= 0.12.0",
        "instrumentation": "opentelemetry-instrumentation-pika==0.41b0",
    },
    "psycopg2": {
        "library": "psycopg2 >= 2.7.3.1",
        "instrumentation": "opentelemetry-instrumentation-psycopg2==0.41b0",
    },
    "pymemcache": {
        "library": "pymemcache >= 1.3.5, < 5",
        "instrumentation": "opentelemetry-instrumentation-pymemcache==0.41b0",
    },
    "pymongo": {
        "library": "pymongo >= 3.1, < 5.0",
        "instrumentation": "opentelemetry-instrumentation-pymongo==0.41b0",
    },
    "PyMySQL": {
        "library": "PyMySQL < 2",
        "instrumentation": "opentelemetry-instrumentation-pymysql==0.41b0",
    },
    "pyramid": {
        "library": "pyramid >= 1.7",
        "instrumentation": "opentelemetry-instrumentation-pyramid==0.41b0",
    },
    "redis": {
        "library": "redis >= 2.6",
        "instrumentation": "opentelemetry-instrumentation-redis==0.41b0",
    },
    "remoulade": {
        "library": "remoulade >= 0.50",
        "instrumentation": "opentelemetry-instrumentation-remoulade==0.41b0",
    },
    "requests": {
        "library": "requests ~= 2.0",
        "instrumentation": "opentelemetry-instrumentation-requests==0.41b0",
    },
    "scikit-learn": {
        "library": "scikit-learn ~= 0.24.0",
        "instrumentation": "opentelemetry-instrumentation-sklearn==0.41b0",
    },
    "sqlalchemy": {
        "library": "sqlalchemy",
        "instrumentation": "opentelemetry-instrumentation-sqlalchemy==0.41b0",
    },
    "starlette": {
        "library": "starlette ~= 0.13.0",
        "instrumentation": "opentelemetry-instrumentation-starlette==0.41b0",
    },
    "psutil": {
        "library": "psutil >= 5",
        "instrumentation": "opentelemetry-instrumentation-system-metrics==0.41b0",
    },
    "tornado": {
        "library": "tornado >= 5.1.1",
        "instrumentation": "opentelemetry-instrumentation-tornado==0.41b0",
    },
    "tortoise-orm": {
        "library": "tortoise-orm >= 0.17.0",
        "instrumentation": "opentelemetry-instrumentation-tortoiseorm==0.41b0",
    },
    "pydantic": {
        "library": "pydantic >= 1.10.2",
        "instrumentation": "opentelemetry-instrumentation-tortoiseorm==0.41b0",
    },
    "urllib3": {
        "library": "urllib3 >= 1.0.0, < 3.0.0",
        "instrumentation": "opentelemetry-instrumentation-urllib3==0.41b0",
    },
}
default_instrumentations = [
    "opentelemetry-instrumentation-aws-lambda==0.41b0",
    "opentelemetry-instrumentation-dbapi==0.41b0",
    "opentelemetry-instrumentation-logging==0.41b0",
    "opentelemetry-instrumentation-sqlite3==0.41b0",
    "opentelemetry-instrumentation-urllib==0.41b0",
    "opentelemetry-instrumentation-wsgi==0.41b0",
]



from bustoken import TK_ELECTRONIC_PAYMENT_REFUND_CANCELED
from messagebus import (
    DataType,
    Message,
    NakMessage
)
from modelcommon.customexception import SitefFinishPaymentRefundCanceled

from ._CommonProcessor import _CommonProcessor
from model import FinishPaymentData


class FinishPaymentProcessor(_CommonProcessor):
    def parse_data(self, data, event_type):
        return FinishPaymentData(*data.split(';'))

    def call_business(self, finish_payment_data):
        # type: (FinishPaymentData) -> str
        return self.payment_router.finish_all_payments(finish_payment_data)

    def format_exception(self, message_bus, message, event, input_obj, exception):
        exception_message = self._get_message(exception=exception)
        message_bus.reply_message(message, exception_message)

    @staticmethod
    def _get_message(
        exception   # type: BaseException
    ):
        # type: (...) -> Message

        if isinstance(exception, type(SitefFinishPaymentRefundCanceled)):
            return Message(TK_ELECTRONIC_PAYMENT_REFUND_CANCELED, DataType.param, str(exception))

        return NakMessage(exception.message)



# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

# Copyright (C) 2012-2017 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""DNS GENERATE range conversion."""

import dns

def from_text(text):
    """Convert the text form of a range in a ``$GENERATE`` statement to an
    integer.

    *text*, a ``str``, the textual range in ``$GENERATE`` form.

    Returns a tuple of three ``int`` values ``(start, stop, step)``.
    """

    # TODO, figure out the bounds on start, stop and step.
    step = 1
    cur = ''
    state = 0
    # state   0 1 2 3 4
    #         x - y / z

    if text and text[0] == '-':
        raise dns.exception.SyntaxError("Start cannot be a negative number")

    for c in text:
        if c == '-' and state == 0:
            start = int(cur)
            cur = ''
            state = 2
        elif c == '/':
            stop = int(cur)
            cur = ''
            state = 4
        elif c.isdigit():
            cur += c
        else:
            raise dns.exception.SyntaxError("Could not parse %s" % (c))

    if state in (1, 3):
        raise dns.exception.SyntaxError()

    if state == 2:
        stop = int(cur)

    if state == 4:
        step = int(cur)

    assert step >= 1
    assert start >= 0
    assert start <= stop
    # TODO, can start == stop?

    return (start, stop, step)



# tva.py - functions for handling French TVA numbers
# coding: utf-8
#
# Copyright (C) 2012-2017 Arthur de Jong
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""n° TVA (taxe sur la valeur ajoutée, French VAT number).

The n° TVA (Numéro d'identification à la taxe sur la valeur ajoutée) is the
SIREN (Système d’Identification du Répertoire des Entreprises) prefixed by
two digits. In old style numbers the two digits are numeric, with new
style numbers at least one is a alphabetic.

>>> compact('Fr 40 303 265 045')
'40303265045'
>>> validate('23334175221')
'23334175221'
>>> validate('84 323 140 391')
Traceback (most recent call last):
    ...
InvalidChecksum: ...
>>> validate('K7399859412')  # new-style number
'K7399859412'
>>> validate('4Z123456782')  # new-style number starting with digit
'4Z123456782'
>>> validate('IO334175221')   # the letters cannot by I or O
Traceback (most recent call last):
    ...
InvalidFormat: ...
"""

from stdnum.exceptions import *
from stdnum.fr import siren
from stdnum.util import clean, isdigits


# the valid characters for the first two digits (O and I are missing)
_alphabet = '0123456789ABCDEFGHJKLMNPQRSTUVWXYZ'


def compact(number):
    """Convert the number to the minimal representation. This strips the
    number of any valid separators and removes surrounding whitespace."""
    number = clean(number, ' -.').upper().strip()
    if number.startswith('FR'):
        number = number[2:]
    return number


def validate(number):
    """Check if the number is a valid VAT number. This checks the length,
    formatting and check digit."""
    number = compact(number)
    if not all(x in _alphabet for x in number[:2]):
        raise InvalidFormat()
    if not isdigits(number[2:]):
        raise InvalidFormat()
    if len(number) != 11:
        raise InvalidLength()
    if number[2:5] != '000':
        # numbers from Monaco are valid TVA but not SIREN
        siren.validate(number[2:])
    if isdigits(number):
        # all-numeric digits
        if int(number[:2]) != (int(number[2:] + '12') % 97):
            raise InvalidChecksum()
    else:
        # one of the first two digits isn't a number
        if isdigits(number[0]):
            check = (
                _alphabet.index(number[0]) * 24 +
                _alphabet.index(number[1]) - 10)
        else:
            check = (
                _alphabet.index(number[0]) * 34 +
                _alphabet.index(number[1]) - 100)
        if (int(number[2:]) + 1 + check // 11) % 11 != (check % 11):
            raise InvalidChecksum()
    return number


def is_valid(number):
    """Check if the number is a valid VAT number."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False
